@@ -272,7 +272,7 @@ void help_remove(void)
 	ent = iter_to_help_entry(&sel);
 	switch (ent->type) {
 	case HE_BOUND:
-		if (yes_no_query("Remove selected binding? [y/N]"))
+		if (yes_no_query("Remove selected binding? [y/N]") == UI_QUERY_ANSWER_YES)
 			key_unbind(key_context_names[ent->binding->ctx],
 					ent->binding->key->name, 0);
 		break;
