@@ -26,12 +26,12 @@
 #include <AudioUnit/AudioUnit.h>
 #include <CoreAudio/CoreAudio.h>
 
-#include "debug.h"
-#include "op.h"
-#include "mixer.h"
-#include "sf.h"
-#include "utils.h"
-#include "xmalloc.h"
+#include "../debug.h"
+#include "../op.h"
+#include "../mixer.h"
+#include "../sf.h"
+#include "../utils.h"
+#include "../xmalloc.h"
 
 
 /////////////////////////////////////////////////////////////////////////////
@@ -74,13 +74,13 @@
  */
 
 /*
- * The text above constitutes the entire PortAudio license; however, 
+ * The text above constitutes the entire PortAudio license; however,
  * the PortAudio community also makes the following non-binding requests:
  *
  * Any person wishing to distribute modifications to the Software is
  * requested to send the modifications to the original developer so that
- * they can be incorporated into the canonical version. It is also 
- * requested that these non-binding requests be included along with the 
+ * they can be incorporated into the canonical version. It is also
+ * requested that these non-binding requests be included along with the
  * license above.
  */
 
@@ -246,7 +246,7 @@ static AudioStreamBasicDescription coreaudio_format_description;
 static AudioUnit coreaudio_audio_unit = NULL;
 static UInt32 coreaudio_buffer_frame_size = 1;
 static coreaudio_ring_buffer_t coreaudio_ring_buffer = {0, 0, 0, 0, 0, NULL};
-static UInt32 coreaudio_stero_channels[2];
+static UInt32 coreaudio_stereo_channels[2];
 static int coreaudio_mixer_pipe_in = 0;
 static int coreaudio_mixer_pipe_out = 0;
 
@@ -317,7 +317,7 @@ static AudioDeviceID coreaudio_find_device(const char *dev_name)
 	err = AudioObjectGetPropertyData(kAudioObjectSystemObject,
 					 &aopa,
 					 0,
-					 NULL, 
+					 NULL,
 					 &property_size,
 					 devices);
 	if (err != noErr)
@@ -573,7 +573,7 @@ static OSStatus coreaudio_init_audio_unit(AudioUnit *au,
 					  OSType os_type,
 					  AudioDeviceID dev_id)
 {
-	OSStatus err;	
+	OSStatus err;
 	AudioComponentDescription comp_desc = {
 		kAudioUnitType_Output,
 		os_type,
@@ -609,7 +609,7 @@ static OSStatus coreaudio_start_audio_unit(AudioUnit *au,
 					   int *frame_size,
 					   AudioStreamBasicDescription desc)
 {
-	
+
 	OSStatus err;
 	err = AudioUnitSetProperty(*au,
 				   kAudioUnitProperty_StreamFormat,
@@ -736,7 +736,7 @@ static OSStatus coreaudio_get_device_stereo_channels(AudioDeviceID dev_id, UInt3
 						  &size,
 						  channels);
 	return err;
-} 
+}
 
 static int coreaudio_mixer_set_volume(int l, int r)
 {
@@ -751,7 +751,7 @@ static int coreaudio_mixer_set_volume(int l, int r)
 		AudioObjectPropertyAddress aopa = {
 			.mSelector	= kAudioDevicePropertyVolumeScalar,
 			.mScope		= kAudioObjectPropertyScopeOutput,
-			.mElement	= coreaudio_stero_channels[i]
+			.mElement	= coreaudio_stereo_channels[i]
 		};
 
 		UInt32 size = sizeof(vol[i]);
@@ -778,7 +778,7 @@ static int coreaudio_mixer_get_volume(int *l, int *r)
 		AudioObjectPropertyAddress aopa = {
 			.mSelector	= kAudioDevicePropertyVolumeScalar,
 			.mScope		= kAudioObjectPropertyScopeOutput,
-			.mElement	= coreaudio_stero_channels[i]
+			.mElement	= coreaudio_stereo_channels[i]
 		};
 		UInt32 size = sizeof(vol[i]);
 		err |= AudioObjectGetPropertyData(coreaudio_device_id,
@@ -808,7 +808,7 @@ static int coreaudio_mixer_get_volume(int *l, int *r)
 static int coreaudio_mixer_open(int *volume_max)
 {
 	*volume_max = coreaudio_max_volume;
-	OSStatus err = coreaudio_get_device_stereo_channels(coreaudio_device_id, coreaudio_stero_channels);
+	OSStatus err = coreaudio_get_device_stereo_channels(coreaudio_device_id, coreaudio_stereo_channels);
 	if (err != noErr) {
 		d_print("Cannot get channel information: %d\n", err);
 		errno = ENODEV;
@@ -818,7 +818,7 @@ static int coreaudio_mixer_open(int *volume_max)
 		AudioObjectPropertyAddress aopa = {
 			.mSelector	= kAudioDevicePropertyVolumeScalar,
 			.mScope		= kAudioObjectPropertyScopeOutput,
-			.mElement	= coreaudio_stero_channels[i]
+			.mElement	= coreaudio_stereo_channels[i]
 		};
 		err |= AudioObjectAddPropertyListener(coreaudio_device_id,
 						      &aopa,
@@ -841,9 +841,9 @@ static int coreaudio_mixer_close(void)
 		AudioObjectPropertyAddress aopa = {
 			.mSelector	= kAudioDevicePropertyVolumeScalar,
 			.mScope		= kAudioObjectPropertyScopeOutput,
-			.mElement	= coreaudio_stero_channels[i]
+			.mElement	= coreaudio_stereo_channels[i]
 		};
-	
+
 		err |= AudioObjectRemovePropertyListener(coreaudio_device_id,
 							 &aopa,
 							 coreaudio_device_volume_change_listener,
