@@ -84,6 +84,9 @@ int mouse = 0;
 int mpris = 1;
 int time_show_leading_zero = 1;
 int start_view = TREE_VIEW;
+int stop_after_queue = 0;
+int tree_width_percent = 33;
+int tree_width_max = 0;
 
 int colors[NR_COLORS] = {
 	-1,
@@ -469,6 +472,34 @@ static void set_status_display_program(void *data, const char *buf)
 		status_display_program = expand_filename(buf);
 }
 
+static void get_tree_width_percent(void *data, char *buf, size_t size)
+{
+	buf_int(buf, tree_width_percent, size);
+}
+
+static void set_tree_width_percent(void *data, const char *buf)
+{
+	int percent;
+
+	if (parse_int(buf, 1, 100, &percent))
+		tree_width_percent = percent;
+	update_size();
+}
+
+static void get_tree_width_max(void *data, char *buf, size_t size)
+{
+	buf_int(buf, tree_width_max, size);
+}
+
+static void set_tree_width_max(void *data, const char *buf)
+{
+	int cols;
+
+	if (parse_int(buf, 0, 9999, &cols))
+		tree_width_max = cols;
+	update_size();
+}
+
 /* }}} */
 
 /* callbacks for toggle options {{{ */
@@ -524,6 +555,24 @@ static void toggle_continue(void *data)
 	update_statusline();
 }
 
+static void get_continue_album(void *data, char *buf, size_t size)
+{
+	strscpy(buf, bool_names[player_cont_album], size);
+}
+
+static void set_continue_album(void *data, const char *buf)
+{
+	if (!parse_bool(buf, &player_cont_album))
+		return;
+	update_statusline();
+}
+
+static void toggle_continue_album(void *data)
+{
+	player_cont_album ^= 1;
+	update_statusline();
+}
+
 static void get_repeat_current(void *data, char *buf, size_t size)
 {
 	strscpy(buf, bool_names[player_repeat_current], size);
@@ -1129,6 +1178,21 @@ static void set_lib_add_filter(void *data, const char *buf)
 	lib_set_add_filter(expr);
 }
 
+static void get_stop_after_queue(void *data, char *buf, size_t size)
+{
+	strscpy(buf, bool_names[stop_after_queue], size);
+}
+
+static void set_stop_after_queue(void *data, const char *buf)
+{
+	parse_bool(buf, &stop_after_queue);
+}
+
+static void toggle_stop_after_queue(void *data)
+{
+	stop_after_queue ^= 1;
+}
+
 /* }}} */
 
 /* special callbacks (id set) {{{ */
@@ -1328,6 +1392,7 @@ static const struct {
 	DN(rewind_offset)
 	DT(confirm_run)
 	DT(continue)
+	DT(continue_album)
 	DT(smart_artist_sort)
 	DN(id3_default_charset)
 	DN(icecast_default_charset)
@@ -1365,6 +1430,9 @@ static const struct {
 	DT(time_show_leading_zero)
 	DN(lib_add_filter)
 	DN(start_view)
+	DT(stop_after_queue)
+	DN(tree_width_percent)
+	DN(tree_width_max)
 	{ NULL, NULL, NULL, NULL, 0 }
 };
 
