@@ -148,6 +148,9 @@ extern int mouse;
 extern int mpris;
 extern int time_show_leading_zero;
 extern int start_view;
+extern int stop_after_queue;
+extern int tree_width_percent;
+extern int tree_width_max;
 
 extern const char * const aaa_mode_names[];
 extern const char * const view_names[NR_VIEWS + 1];
