@@ -1,6 +1,7 @@
-/* 
+/*
+ * Copyright 2008-2013 Various Authors
  * Copyright 2004 Timo Hirvonen
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation; either version 2 of the
@@ -10,28 +11,36 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef _SEARCH_MODE_H
-#define _SEARCH_MODE_H
+#ifndef CMUS_SEARCH_MODE_H
+#define CMUS_SEARCH_MODE_H
 
 #include "search.h"
 #include "uchar.h"
 
+#if defined(__sun__)
+#include <ncurses.h>
+#else
+#include <curses.h>
+#endif
+
 extern char *search_str;
 extern enum search_direction search_direction;
 
 /* //WORDS or ??WORDS search mode */
 extern int search_restricted;
 
-extern void search_mode_ch(uchar ch);
-extern void search_mode_key(int key);
-extern void search_mode_init(void);
-extern void search_mode_exit(void);
+void search_mode_ch(uchar ch);
+void search_mode_escape(int c);
+void search_mode_key(int key);
+void search_mode_mouse(MEVENT *event);
+void search_mode_init(void);
+void search_mode_exit(void);
+
+void search_text(const char *text, int restricted, int beginning);
 
 #endif
