@@ -24,6 +24,7 @@
 #include "uchar.h"
 #include "xmalloc.h"
 #include "debug.h"
+#include "misc.h"
 
 #include <string.h>
 
@@ -51,7 +52,7 @@ int simple_track_search_get_current(void *data, struct iter *iter)
 	return window_get_sel(data, iter);
 }
 
-int simple_track_search_matches(void *data, struct iter *iter, const char *text)
+int _simple_track_search_matches(struct iter *iter, const char *text)
 {
 	unsigned int flags = TI_MATCH_TITLE;
 	struct simple_track *track = iter_to_simple_track(iter);
@@ -59,11 +60,15 @@ int simple_track_search_matches(void *data, struct iter *iter, const char *text)
 	if (!search_restricted)
 		flags |= TI_MATCH_ARTIST | TI_MATCH_ALBUM | TI_MATCH_ALBUMARTIST;
 
-	if (!track_info_matches(track->info, text, flags))
-		return 0;
+	return track_info_matches(track->info, text, flags);
+}
 
-	window_set_sel(data, iter);
-	return 1;
+int simple_track_search_matches(void *data, struct iter *iter, const char *text)
+{
+	int rc = _simple_track_search_matches(iter, text);
+	if (rc)
+		window_set_sel(data, iter);
+	return rc;
 }
 
 void shuffle_insert(struct rb_root *root, struct shuffle_track *previous, struct shuffle_track *next)
@@ -87,7 +92,7 @@ void shuffle_insert(struct rb_root *root, struct shuffle_track *previous, struct
 }
 
 struct shuffle_track *shuffle_list_get_next(struct rb_root *root, struct shuffle_track *cur,
-		int (*filter)(const struct simple_track *))
+		int (*filter_callback)(const struct simple_track *))
 {
 	struct rb_node *node;
 
@@ -99,7 +104,7 @@ again:
 	while (node) {
 		struct shuffle_track *track = tree_node_to_shuffle_track(node);
 
-		if (filter((struct simple_track *)track))
+		if (filter_callback((struct simple_track *)track))
 			return track;
 		node = rb_next(node);
 	}
@@ -113,7 +118,7 @@ again:
 }
 
 struct shuffle_track *shuffle_list_get_prev(struct rb_root *root, struct shuffle_track *cur,
-		int (*filter)(const struct simple_track *))
+		int (*filter_callback)(const struct simple_track *))
 {
 	struct rb_node *node;
 
@@ -125,7 +130,7 @@ again:
 	while (node) {
 		struct shuffle_track *track = tree_node_to_shuffle_track(node);
 
-		if (filter((struct simple_track *)track))
+		if (filter_callback((struct simple_track *)track))
 			return track;
 		node = rb_prev(node);
 	}
@@ -139,7 +144,7 @@ again:
 }
 
 struct simple_track *simple_list_get_next(struct list_head *head, struct simple_track *cur,
-		int (*filter)(const struct simple_track *))
+		int (*filter_callback)(const struct simple_track *))
 {
 	struct list_head *item;
 
@@ -151,7 +156,7 @@ again:
 	while (item != head) {
 		struct simple_track *track = to_simple_track(item);
 
-		if (filter(track))
+		if (filter_callback(track))
 			return track;
 		item = item->next;
 	}
@@ -162,7 +167,7 @@ again:
 }
 
 struct simple_track *simple_list_get_prev(struct list_head *head, struct simple_track *cur,
-		int (*filter)(const struct simple_track *))
+		int (*filter_callback)(const struct simple_track *))
 {
 	struct list_head *item;
 
@@ -174,7 +179,7 @@ again:
 	while (item != head) {
 		struct simple_track *track = to_simple_track(item);
 
-		if (filter(track))
+		if (filter_callback(track))
 			return track;
 		item = item->prev;
 	}
@@ -250,6 +255,37 @@ void sorted_list_remove_track(struct list_head *head, struct rb_root *tree_root,
 	list_del(&track->node);
 }
 
+void rand_list_rebuild(struct list_head *head, struct rb_root *tree_root)
+{
+	struct list_head *item, *tmp;
+	struct rb_root tmp_tree = RB_ROOT;
+	struct simple_track **track_array;
+	static const sort_key_t empty_sort_keys[] = { SORT_INVALID };
+
+	unsigned int len = 0, track_cnt = 0;
+
+	list_for_each(item, head) {
+		len++;
+	}
+	track_array = xmalloc(len * sizeof(track_array[0]));
+
+	LIST_HEAD(tmp_head);
+	list_for_each_safe(item, tmp, head) {
+		struct simple_track *track = to_simple_track(item);
+		sorted_list_remove_track(head, tree_root, track);
+		track_array[track_cnt] = track;
+		track_cnt++;
+	}
+	shuffle_array(track_array, len, sizeof(track_array[0]));
+	for (unsigned int i=0; i<len; i++) {
+		sorted_list_add_track(&tmp_head, &tmp_tree, track_array[i], empty_sort_keys, 0);
+	}
+	free(track_array);
+
+	tree_root->rb_node = tmp_tree.rb_node;
+	_list_add(head, tmp_head.prev, tmp_head.next);
+}
+
 void sorted_list_rebuild(struct list_head *head, struct rb_root *tree_root, const sort_key_t *keys)
 {
 	struct list_head *item, *tmp;
@@ -349,8 +385,8 @@ void list_add_rand(struct list_head *head, struct list_head *node, int nr)
 	}
 }
 
-int simple_list_for_each_marked(struct list_head *head,
-		int (*cb)(void *data, struct track_info *ti), void *data, int reverse)
+int simple_list_for_each_marked(struct list_head *head, track_info_cb cb,
+		void *data, int reverse)
 {
 	struct simple_track *t;
 	int rc = 0;
@@ -374,3 +410,24 @@ int simple_list_for_each_marked(struct list_head *head,
 	}
 	return rc;
 }
+
+int simple_list_for_each(struct list_head *head, track_info_cb cb, void *data,
+		int reverse)
+{
+	struct simple_track *t;
+	int rc = 0;
+
+	if (reverse) {
+		list_for_each_entry_reverse(t, head, node) {
+			if ((rc = cb(data, t->info)))
+				break;
+		}
+	} else {
+		list_for_each_entry(t, head, node) {
+			if ((rc = cb(data, t->info)))
+				break;
+		}
+	}
+
+	return rc;
+}
