@@ -1,6 +1,7 @@
-/* 
+/*
+ * Copyright 2008-2013 Various Authors
  * Copyright 2004-2005 Timo Hirvonen
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation; either version 2 of the
@@ -10,21 +11,26 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "uchar.h"
 #include "compiler.h"
+#include "gbuf.h"
+#include "utils.h" /* N_ELEMENTS */
+#include "ui_curses.h" /* using_utf8, charset */
+#include "convert.h"
 
 #include <stdlib.h>
 #include <string.h>
+#include <wchar.h>
 #include <wctype.h>
 #include <ctype.h>
 
+#include "unidecomp.h"
+
 const char hex_tab[16] = "0123456789abcdef";
 
 /*
@@ -75,6 +81,19 @@ static const signed char len_tab[256] = {
 	-1, -1, -1, -1, -1, -1, -1, -1
 };
 
+/* fault-tolerant equivalent to len_tab, from glib */
+static const char utf8_skip_data[256] = {
+	1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
+	1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
+	1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
+	1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
+	1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
+	1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
+	2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
+	3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,4,4,4,4,4,4,4,4,5,5,5,5,6,6,1,1
+};
+const char * const utf8_skip = utf8_skip_data;
+
 /* index is length of the UTF-8 sequence - 1 */
 static int min_val[4] = { 0x000000, 0x000080, 0x000800, 0x010000 };
 static int max_val[4] = { 0x00007f, 0x0007ff, 0x00ffff, 0x10ffff };
@@ -116,10 +135,18 @@ int u_is_valid(const char *str)
 	return 1;
 }
 
-int u_strlen(const char *str)
+size_t u_strlen(const char *str)
+{
+	size_t len;
+	for (len = 0; *str; len++)
+		str = u_next_char(str);
+	return len;
+}
+
+size_t u_strlen_safe(const char *str)
 {
 	const unsigned char *s = (const unsigned char *)str;
-	int len = 0;
+	size_t len = 0;
 
 	while (*s) {
 		int l = len_tab[*s];
@@ -154,13 +181,21 @@ int u_char_width(uchar u)
 	if (unlikely(u < 0x20))
 		goto control;
 
+	/* Combining Diacritical Marks */
+	if (u >= 0x300U && u <= 0x36fU)
+		goto zero;
+
 	if (u < 0x1100U)
 		goto narrow;
 
-	/* Hangul Jamo init. consonants */ 
+	/* Hangul Jamo init. consonants */
 	if (u <= 0x115fU)
 		goto wide;
 
+	/* Zero-width characters */
+	if (u == 0x200bU || u == 0x200cU || u == 0x200dU)
+		goto zero;
+
 	/* angle brackets */
 	if (u == 0x2329U || u == 0x232aU)
 		goto wide;
@@ -198,10 +233,18 @@ int u_char_width(uchar u)
 	if (u >= 0xff00U && u <= 0xff60U)
 		goto wide;
 
+	/* Halfwidth Forms */
+	if (u >= 0xff61U && u <= 0xffdfU)
+		goto narrow;
+
 	/* Fullwidth Forms */
 	if (u >= 0xffe0U && u <= 0xffe6U)
 		goto wide;
 
+	/* Halfwidth Forms */
+	if (u >= 0xffe8U && u <= 0xffeeU)
+		goto narrow;
+
 	/* CJK extra stuff */
 	if (u >= 0x20000U && u <= 0x2fffdU)
 		goto wide;
@@ -213,6 +256,8 @@ int u_char_width(uchar u)
 	/* invalid bytes in unicode stream are rendered "<xx>" */
 	if (u & U_INVALID_MASK)
 		goto invalid;
+zero:
+	return 0;
 narrow:
 	return 1;
 wide:
@@ -232,9 +277,7 @@ int u_str_width(const char *str)
 	int idx = 0, w = 0;
 
 	while (str[idx]) {
-		uchar u;
-
-		u_get_char(str, &idx, &u);
+		uchar u = u_get_char(str, &idx);
 		w += u_char_width(u);
 	}
 	return w;
@@ -244,10 +287,9 @@ int u_str_nwidth(const char *str, int len)
 {
 	int idx = 0;
 	int w = 0;
-	uchar u;
 
 	while (len > 0) {
-		u_get_char(str, &idx, &u);
+		uchar u = u_get_char(str, &idx);
 		if (u == 0)
 			break;
 		w += u_char_width(u);
@@ -256,6 +298,18 @@ int u_str_nwidth(const char *str, int len)
 	return w;
 }
 
+char *u_strchr(const char *str, uchar uch)
+{
+	int idx = 0;
+
+	while (str[idx]) {
+		uchar u = u_get_char(str, &idx);
+		if (uch == u)
+			return (char *) (str + idx);
+	}
+	return NULL;
+}
+
 void u_prev_char_pos(const char *str, int *idx)
 {
 	const unsigned char *s = (const unsigned char *)str;
@@ -265,7 +319,7 @@ void u_prev_char_pos(const char *str, int *idx)
 	ch = s[--i];
 	len = len_tab[ch];
 	if (len != 0) {
-		/* start of byte sequence or invelid uchar */
+		/* start of byte sequence or invalid uchar */
 		goto one;
 	}
 
@@ -301,34 +355,41 @@ one:
 	return;
 }
 
-void u_get_char(const char *str, int *idx, uchar *uch)
+uchar u_get_char(const char *str, int *idx)
 {
 	const unsigned char *s = (const unsigned char *)str;
-	int len, i = *idx;
+	int len, i, x = 0;
 	uchar ch, u;
 
-	ch = s[i++];
+	if (idx)
+		s += *idx;
+	else
+		idx = &x;
+	ch = s[0];
+
+	/* ASCII optimization */
+	if (ch < 128) {
+		*idx += 1;
+		return ch;
+	}
+
 	len = len_tab[ch];
 	if (unlikely(len < 1))
 		goto invalid;
 
-	len--;
-	u = ch & first_byte_mask[len];
-	while (len > 0) {
-		ch = s[i++];
+	u = ch & first_byte_mask[len - 1];
+	for (i = 1; i < len; i++) {
+		ch = s[i];
 		if (unlikely(len_tab[ch] != 0))
 			goto invalid;
 		u = (u << 6) | (ch & 0x3f);
-		len--;
 	}
-	*idx = i;
-	*uch = u;
-	return;
+	*idx += len;
+	return u;
 invalid:
-	i = *idx;
-	u = s[i++];
-	*uch = u | U_INVALID_MASK;
-	*idx = i;
+	*idx += 1;
+	u = s[0];
+	return u | U_INVALID_MASK;
 }
 
 void u_set_char_raw(char *str, int *idx, uchar uch)
@@ -367,7 +428,7 @@ void u_set_char_raw(char *str, int *idx, uchar uch)
  * Printing functions, these lose information
  */
 
-void u_set_char(char *str, int *idx, uchar uch)
+void u_set_char(char *str, size_t *idx, uchar uch)
 {
 	int i = *idx;
 
@@ -415,15 +476,16 @@ invalid:
 	}
 }
 
-int u_copy_chars(char *dst, const char *src, int *width)
+size_t u_copy_chars(char *dst, const char *src, int *width)
 {
 	int w = *width;
-	int si = 0, di = 0;
+	int si = 0;
+	size_t di = 0;
 	int cw;
 	uchar u;
 
 	while (w > 0) {
-		u_get_char(src, &si, &u);
+		u = u_get_char(src, &si);
 		if (u == 0)
 			break;
 
@@ -449,15 +511,56 @@ int u_copy_chars(char *dst, const char *src, int *width)
 	return di;
 }
 
+int u_to_ascii(char *dst, const char *src, int len)
+{
+	int i, idx = 0;
+	for (i = 0; i < len && src[idx]; i++) {
+		uchar u = u_get_char(src, &idx);
+		dst[i] = (u < 128) ? u : '?';
+	}
+	return i;
+}
+
+void u_to_utf8(char *dst, const char *src)
+{
+	int s = 0;
+	size_t d = 0;
+	uchar u;
+	do {
+		u = u_get_char(src, &s);
+		u_set_char(dst, &d, u);
+	} while (u!=0);
+}
+
+int u_print_size(uchar uch)
+{
+	int s = u_char_size(uch);
+	/* control characters and invalid unicode set as <XX> */
+	if (uch < 0x0000001fU && uch != 0){
+		return 4;
+	}
+	return s;
+}
+
+int u_str_print_size(const char *str)
+{
+	int l = 0;
+	int idx = 0;
+	uchar u;
+	do {
+		u = u_get_char(str, &idx);
+		l += u_print_size(u);
+	} while (u!=0);
+	return l;
+}
+
 int u_skip_chars(const char *str, int *width)
 {
 	int w = *width;
 	int idx = 0;
 
 	while (w > 0) {
-		uchar u;
-
-		u_get_char(str, &idx, &u);
+		uchar u = u_get_char(str, &idx);
 		w -= u_char_width(u);
 	}
 	/* add 1..3 if skipped 'too much' (the last char was double width or invalid (<xx>)) */
@@ -465,79 +568,158 @@ int u_skip_chars(const char *str, int *width)
 	return idx;
 }
 
+/*
+ * Case-folding functions
+ */
+
+static inline uchar u_casefold_char(uchar ch)
+{
+	/* faster lookup for for A-Z, rest of ASCII unaffected */
+	if (ch < 0x0041)
+		return ch;
+	if (ch <= 0x005A)
+		return ch + 0x20;
+#if defined(_WIN32) || defined(__STDC_ISO_10646__) || defined(__APPLE__)
+	if (ch < 128)
+		return ch;
+	ch = towlower(ch);
+#endif
+	return ch;
+}
+
+char *u_casefold(const char *str)
+{
+	GBUF(out);
+	int i = 0;
+
+	while (str[i]) {
+		char buf[4];
+		int buflen = 0;
+		uchar ch = u_get_char(str, &i);
+
+		ch = u_casefold_char(ch);
+		u_set_char_raw(buf, &buflen, ch);
+		gbuf_add_bytes(&out, buf, buflen);
+	}
+
+	return gbuf_steal(&out);
+}
+
 /*
  * Comparison functions
  */
 
-static inline int chcasecmp(int a, int b)
+int u_strcase_equal(const char *a, const char *b)
 {
-	return towupper(a) - towupper(b);
-}
+	int ai = 0, bi = 0;
 
-int u_strcasecmp(const char *a, const char *b)
-{
-	int ai = 0;
-	int bi = 0;
-	int res;
-
-	do {
+	while (a[ai]) {
 		uchar au, bu;
 
-		u_get_char(a, &ai, &au);
-		u_get_char(b, &bi, &bu);
-		res = chcasecmp(au, bu);
-		if (res)
-			break;
-		if (au == 0) {
-			/* bu is 0 too */
-			break;
-		}
-	} while (1);
-	return res;
-}
+		au = u_get_char(a, &ai);
+		bu = u_get_char(b, &bi);
 
-int u_strncasecmp(const char *a, const char *b, int len)
-{
-	int ai = 0;
-	int bi = 0;
-
-	while (len > 0) {
-		uchar au, bu;
-		int res;
-
-		u_get_char(a, &ai, &au);
-		u_get_char(b, &bi, &bu);
-		res = chcasecmp(au, bu);
-		if (res)
-			return res;
-		if (au == 0) {
-			/* bu is 0 too */
+		if (u_casefold_char(au) != u_casefold_char(bu))
 			return 0;
-		}
-		len--;
 	}
-	return 0;
+
+	return b[bi] ? 0 : 1;
 }
 
-char *u_strcasestr(const char *haystack, const char *needle)
+static uchar get_base_from_composed(uchar ch)
+{
+	int begin = 0;
+	int end = N_ELEMENTS(unidecomp_map);
+
+	if (ch < unidecomp_map[begin].composed || ch > unidecomp_map[end - 1].composed)
+		return ch;
+
+	/* binary search */
+	while (1) {
+		int half = (begin + end) / 2;
+		if (ch == unidecomp_map[half].composed)
+			return unidecomp_map[half].base;
+		else if (half == begin)
+			break;
+		else if (ch > unidecomp_map[half].composed)
+			begin = half;
+		else
+			end = half;
+	}
+	return ch;
+}
+
+static inline int do_u_strncase_equal(const char *a, const char *b, size_t len, int only_base_chars)
+{
+	int ai = 0, bi = 0;
+	size_t i;
+
+	for (i = 0; i < len; i++) {
+		uchar au, bu;
+
+		au = u_get_char(a, &ai);
+		bu = u_get_char(b, &bi);
+
+		if (only_base_chars) {
+			au = get_base_from_composed(au);
+			bu = get_base_from_composed(bu);
+		}
+
+		if (u_casefold_char(au) != u_casefold_char(bu))
+			return 0;
+	}
+
+	return 1;
+}
+
+int u_strncase_equal(const char *a, const char *b, size_t len)
+{
+	return do_u_strncase_equal(a, b, len, 0);
+}
+
+int u_strncase_equal_base(const char *a, const char *b, size_t len)
+{
+	return do_u_strncase_equal(a, b, len, 1);
+}
+
+static inline char *do_u_strcasestr(const char *haystack, const char *needle, int only_base_chars)
 {
 	/* strlen is faster and works here */
 	int haystack_len = strlen(haystack);
 	int needle_len = u_strlen(needle);
 
 	do {
-		uchar u;
 		int idx;
 
 		if (haystack_len < needle_len)
 			return NULL;
-		if (u_strncasecmp(needle, haystack, needle_len) == 0)
+		if (do_u_strncase_equal(needle, haystack, needle_len, only_base_chars))
 			return (char *)haystack;
 
 		/* skip one char */
 		idx = 0;
-		u_get_char(haystack, &idx, &u);
+		u_get_char(haystack, &idx);
 		haystack += idx;
 		haystack_len -= idx;
 	} while (1);
 }
+
+char *u_strcasestr(const char *haystack, const char *needle)
+{
+	return do_u_strcasestr(haystack, needle, 0);
+}
+
+char *u_strcasestr_base(const char *haystack, const char *needle)
+{
+	return do_u_strcasestr(haystack, needle, 1);
+}
+
+char *u_strcasestr_filename(const char *haystack, const char *needle)
+{
+	char *r = NULL, *ustr = NULL;
+	if (!using_utf8 && utf8_encode(haystack, charset, &ustr) == 0)
+		haystack = ustr;
+	r = u_strcasestr_base(haystack, needle);
+	free(ustr);
+	return r;
+}
